#-----------------------------------------------------------------------
# Modified 2003 Boris Schfer <menc@noid-project>
# for the Video::Menc project
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This module is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::Menc::Content;

use base Video::Menc::Base;

use Video::Menc::ContentProbe;
use Video::Menc::Title;

use Carp;
use strict;

sub project          { shift->{project}      }
sub titles           { shift->{titles}       } # href/undef
sub probe_result     { shift->{probe_result} }

sub set_titles       { shift->{titles}       = $_[1] }
sub set_probe_result { shift->{probe_result} = $_[1] }

sub set_project  {
  my $self = shift;
  my ($project) = @_;

  $self->{project} = $project;
  
  return if not $self->titles;

  foreach my $title ( values %{$self->titles} ) {
    $title->set_project ($project);
  }
  
  return $project;
}

sub new {
  my $class = shift;
  my %par = @_;
  my  ($project) = @par{'project'};

  my $self = {
    project     => $project,
    titles      => undef,
    probe_result   => undef,
  };
  
  return bless $self, $class;
}

sub read_title_listing {
  my $self = shift;

  my $titlecount = $self->probe_result->titles;
  my $title;
  my %titles;
  
  my $project = $self->project;
  my $dvd_device = $project->dvd_device;
  
  for (my $i=1; $i <= $titlecount; $i++) {
      $title = $titles{$i};
    $title ||= Video::Menc::Title->new (
      dvdtrack => $i,
      project => $project
    );
    $titles{$i} ||= $title;
    
    $title->set_type("dvd");
    $title->set_postcmd(" dvd://$i -dvd-device \"$dvd_device\" ");
  }  

  # story Title objects
  $self->set_titles (\%titles);

   1;
}

sub get_titles {
  my $self = shift;
  
  $self->read_title_listing if not $self->titles;
  
  my @titles = sort { $a->dvdtrack <=> $b->dvdtrack } values %{$self->titles};
  
  return \@titles;
}

#---------------------------------------------------------------------
# Methods for Probing DVD
#---------------------------------------------------------------------

sub get_dvd_content_command {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($dvdtrack, $audioid, $notcprobe) = 
    @par{'dvdtrack', 'audioid', 'notcprobe'};
    
  my $dvd_device    = $self->project->dvd_device;
  
  my $audio = "-aid $audioid" if ($audioid);
  my $tcprobe = "tcprobe -i \"$dvd_device\" -T $dvdtrack";
  $tcprobe = "" if ($notcprobe);
  
#print "CMD = 'mplayer -identify -v -vo null -ao null -frames 1 -dvd-device \"$dvd_device\" ".
#      "dvd://$dvdtrack $audio ; $tcprobe'\n";  
  return "mplayer -identify -v -vo null -ao null -frames 1 -dvd-device \"$dvd_device\" ".
         "dvd://$dvdtrack $audio ; $tcprobe";
}

sub dvd_content_async_start {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($dvdtrack, $audioid, $notcprobe) = 
    @par{'dvdtrack', 'audioid', 'notcprobe'};
        
  return $self->popen (
    command => $self->get_dvd_content_command (
      dvdtrack  => $dvdtrack,
      audioid   => $audioid,
      notcprobe => $notcprobe,
    ),
  );
}

sub dvd_content_async_stop {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($fh, $output, $dvdtrack, $audioid, $notcprobe) = 
    @par{'fh','output', 'dvdtrack', 'audioid', 'notcprobe'};

  $output = "\n\nOutput was:\n\n$output" if $output;

  my $message =   "Error executing:\n\n".
    $self->get_dvd_content_command (
      dvdtrack  => $dvdtrack,
      audioid   => $audioid,
      notcprobe => $notcprobe,
    ).
    $output;

  close $fh or croak ($message);
  
  $self->analyze_dvd_content_output (
    output => $output
  );
  
  1;
}

sub analyze_dvd_content_output {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($output) = @par{'output'};

  $self->set_probe_result (
    Video::Menc::ContentProbe->analyze_dvd (
      probe_output => $output,
    )
  );
  
  1;
}

#---------------------------------------------------------------------
# Methods for Probing File
#---------------------------------------------------------------------

sub get_file_content_command {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($filename, $audioid) = 
    @par{'filename', 'audioid'};
    
  my $audio = "-aid $audioid" if ($audioid >= 0);

print "mplayer -identify -v -vo null -ao null -frames 1 $audio \"$filename\" \n";
  return "mplayer -identify -v -vo null -ao null -frames 1 $audio \"$filename\" ";
}

sub file_content_async_start {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($filename, $audioid) = 
    @par{'filename', 'audioid'};
        
  return $self->popen (
    command => $self->get_file_content_command (
      filename  => $filename,
      audioid   => $audioid,
    ),
  );
}

sub file_content_async_stop {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($fh, $output, $filename, $audioid) = 
    @par{'fh','output', 'filename', 'audioid'};

  $output = "\n\nOutput was:\n\n$output" if $output;

  my $message =   "Error executing:\n\n".
    $self->get_dvd_content_command (
      filename  => $filename,
      audioid   => $audioid,
    ).
    $output;

  close $fh or croak ($message);
  
  $self->analyze_file_content_output (
    output => $output
  );
  
  1;
}

sub analyze_file_content_output {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($output) = @par{'output'};

  $self->set_probe_result (
    Video::Menc::ContentProbe->analyze_file (
      probe_output => $output,
    )
  );
  
  1;
}

1;
