#-----------------------------------------------------------------------
# Copyright (C) 2003 Boris Schfer <menc@noid-project> All Rights Reserved
# 
# This module is part of Video::Menc, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::Menc::ContentProbe;

use base Video::Menc::Base;

use Carp;
use strict;

sub titles           { shift->{titles}       }
sub chapters         { shift->{chapters}     }
sub audio_tracks     { shift->{audio_tracks} }  # lref
sub audio_track      { shift->{audio_track}  }
sub subtitles        { shift->{subtitles}    }
sub abitrate         { shift->{abitrate}     }
sub aformat          { shift->{aformat}      }
sub width            { shift->{width}        }
sub height           { shift->{height}       }
sub fps              { shift->{fps}          }
sub aspect           { shift->{aspect}       }
sub runtime          { shift->{runtime}      }
sub probe_output     { shift->{probe_output} }

sub set_titles       { shift->{titles}       = $_[1] }
sub set_chapters     { shift->{chapters}     = $_[1] }
sub set_audio_tracks { shift->{audio_tracks} = $_[1] }
sub set_audio_track  { shift->{audio_track}  = $_[1] }
sub set_subtitles    { shift->{subtitles}    = $_[1] }
sub set_abitrate     { shift->{abitrate}     = $_[1] }
sub set_aformat      { shift->{aformat}      = $_[1] }
sub set_width        { shift->{width}        = $_[1] }
sub set_height       { shift->{height}       = $_[1] }
sub set_fps          { shift->{fps}          = $_[1] }
sub set_aspect       { shift->{aspect}       = $_[1] }
sub set_runtime      { shift->{runtime}      = $_[1] }
sub set_probe_output { shift->{probe_output} = $_[1] }

sub analyze_dvd {
  my $class = shift;
  my %par = @_;
  my  ($probe_output) = @par{'probe_output'};

  my ($titles, $chapters, $width, $height, $fps, $aspect, $abitrate, $runtime);

  ($titles)   = $probe_output =~ /There\s+are\s+(\d+)\s+titles\s+on\s+this\s+DVD./; 
  ($chapters) = $probe_output =~ /There\s+are\s+(\d+)\s+chapters\s+in\s+this\s+DVD\s+title./;
  ($width)    = $probe_output =~ /ID_VIDEO_WIDTH=(\d+)/;
  ($height)   = $probe_output =~ /ID_VIDEO_HEIGHT=(\d+)/;
  ($fps)      = $probe_output =~ /ID_VIDEO_FPS=([0-9,]+)/;
  ($aspect)   = $probe_output =~ /ID_VIDEO_ASPECT=([0-9,]+)/;
  ($abitrate) = $probe_output =~ /ID_AUDIO_BITRATE=(\d+)/;
  ($runtime)  = $probe_output =~ /title\s+playback\s+time:\s+[^\s]+\s+(\d+)/;

  $fps =~ s/,/\./g;
  $aspect =~ s/,/\./g;

  my (@audio_tracks);
  my $i = 0;
  while ( $probe_output =~ /\[open\]\s+audio\s+stream:\s+([^\n]+)/g ) {
    my $string = $1;

    $string =~ /(\d+)\s+audio\s+format:\s+(.+)\s+language:\s+(.+)\s+aid:\s+(\d+)/;

    my $format = $2;
    my $lang = uc($3);
    my $id = $4;
    
    $audio_tracks[$i] = {
      format => $format,
      lang   => $lang,
      id     => $id
    };
    
    $i++;
  }
  
  my (@subtitles);
  $i = 0;
  while ( $probe_output =~ /ID_SID_([0-9]+)_LANG=([^\n]+)/g ) {
    my $id = $1;
    my $lang = $2;
 
    $subtitles[$i] = {
      id => $id,
      lang => $lang,
    };
    
    $i++;
  }

  my $self = {
    probe_output => $probe_output,
    titles       => $titles,
    chapters     => $chapters,
    audio_tracks => \@audio_tracks,
    subtitles    => \@subtitles,
    width        => $width,
    height       => $height,
    fps          => $fps,
    aspect       => $aspect,
    abitrate     => $abitrate,
    runtime      => $runtime,
  };
  
  return bless $self, $class;
}

sub analyze_file {
  my $class = shift;
  my %par = @_;
  my  ($probe_output) = @par{'probe_output'};

  my ($width, $height, $fps, $aspect, $abitrate, $runtime, $aformat, $ac3);

  ($width)    = $probe_output =~ /ID_VIDEO_WIDTH=(\d+)/;
  ($height)   = $probe_output =~ /ID_VIDEO_HEIGHT=(\d+)/;
  ($fps)      = $probe_output =~ /ID_VIDEO_FPS=([0-9,]+)/;
  ($aspect)   = $probe_output =~ /ID_VIDEO_ASPECT=([0-9,]+)/;
  ($abitrate) = $probe_output =~ /ID_AUDIO_BITRATE=(\d+)/;
  ($aformat)  = $probe_output =~ /ID_AUDIO_CODEC=(.+)/;
  ($ac3)      = $probe_output =~ /AC3:\s+([0-9\.]+)/;
  ($runtime)  = $probe_output =~ /ID_LENGTH=(\d+)/;

  $fps =~ s/,/\./g;
  $aspect =~ s/,/\./g;
  $aformat = uc($aformat);
  $aformat = "AC3 $ac3" if ( $ac3 );

  my (@audio_tracks);
  my $i = 0;
  while ( $probe_output =~ /==\>\s+Found\s+audio\s+stream:\s+(\d+)/g ) {
    $audio_tracks[$i] = {
      id => $1
    };
    
    $i++;
  }

  my $self = {
    probe_output => $probe_output,
    audio_tracks => \@audio_tracks,
    width        => $width,
    height       => $height,
    fps          => $fps,
    aspect       => $aspect,
    abitrate     => $abitrate,
    aformat      => $aformat,
    runtime      => $runtime,
  };
  
  return bless $self, $class;
}

1;
