#-----------------------------------------------------------------------
# Copyright (C) 2003 Boris Schfer <menc@noid-project> All Rights Reserved
# 
# This module is part of Video::Menc, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::Menc::GUI::Project;

use Carp;
use strict;

sub gtk_audio_popup2       { shift->{gtk_audio_popup2}   }  # lref
sub set_gtk_audio_popup2   { shift->{gtk_audio_popup2}   = $_[1] }
sub gtk_audio_popup2_2     { shift->{gtk_audio_popup2_2} }  # lref
sub set_gtk_audio_popup2_2 { shift->{gtk_audio_popup2_2} = $_[1] }

sub file_title             { shift->{file_title}         }  
sub set_file_title         { shift->{file_title}         = $_[1] }

#------------------------------------------------------------------------
# Build RIP Title Tab
#------------------------------------------------------------------------

sub create_title2_tab {
  my $self = shift; $self->trace_in;

  my $vbox = Gtk::VBox->new;
  $vbox->set_border_width(5);
  $vbox->show;
  
  my $frame = Gtk::Frame->new ("File");
  $frame->show;
  $vbox->pack_start ( $frame, 0, 1, 0);

  my $file_vbox = Gtk::VBox->new;
  $file_vbox->set_border_width(5);
  $file_vbox->show;
  $frame->add ( $file_vbox );

  my $hbox = Gtk::HBox->new;
  $hbox->set_border_width(5);
  $hbox->show;
  $file_vbox->pack_start($hbox, 0, 1, 0);
  
  my $label = Gtk::Label->new(" Filename ");
  $label->show;
  $hbox->pack_start($label, 0, 1, 0);

  my $entry = Gtk::Entry->new;
  $entry->set_text ("");
  $entry->set_usize(500, undef);
  $entry->show;
  $entry->signal_connect ("activate",
    sub { $self->use_file }
  );
  $self->adjust_widgets->{filename} = $entry;

  $hbox->pack_start($entry, 0, 1, 0);
    
  my $button = Gtk::Button->new_with_label (" Browse... ");
  $button->show;
  $hbox->pack_start ($button, 1, 1, 0);
  $button->signal_connect ("clicked",
    sub { $self->browse_file }
  );
  
  # Fileinfos & Audio
  $hbox = Gtk::HBox->new;
  $hbox->set_border_width(5);
  $hbox->show;
  $file_vbox->pack_start($hbox, 1, 1, 0);
  
  $label = Gtk::Label->new;
  $label->show;
  $label->set_justify('left');
  
  $hbox->pack_start ( $label, 0, 1, 0);  
  
  $self->adjust_widgets->{fileinfos} = $label;

  my $audio_popup_menu = Gtk::Menu->new;
  $audio_popup_menu->show;
  my $item = Gtk::MenuItem->new ("No Audio");
  $item->show;
  $audio_popup_menu->append($item);
  my $audio_popup = Gtk::OptionMenu->new;
  $audio_popup->show;
  $audio_popup->set_menu($audio_popup_menu);
  $audio_popup->set_usize(200,undef);
  
  $self->set_gtk_audio_popup2 ($audio_popup);
  $hbox->pack_end($audio_popup, 0, 1, 0);

  $label = Gtk::Label->new ("Select Audio Channel ");
  $label->show;
  $hbox->pack_end ($label, 0, 1, 0);
  
  # 2nd Audio
  $hbox = Gtk::HBox->new;
  $hbox->set_border_width(5);
  $hbox->show;
  $file_vbox->pack_start($hbox, 1, 1, 0);  
  
  $audio_popup_menu = Gtk::Menu->new;
  $audio_popup_menu->show;
  $item = Gtk::MenuItem->new ("No Audio");
  $item->show;
  $audio_popup_menu->append($item);
  $audio_popup = Gtk::OptionMenu->new;
  $audio_popup->show;
  $audio_popup->set_menu($audio_popup_menu);
  $audio_popup->set_usize(200,undef);
  
  $self->set_gtk_audio_popup2_2 ($audio_popup);
  $hbox->pack_end($audio_popup, 0, 1, 0);

  $label = Gtk::Label->new ("Select 2nd Audio Channel ");
  $label->show;
  $hbox->pack_end ($label, 0, 1, 0);
  
  # Buttons
  $hbox = Gtk::HBox->new (1);
  $hbox->set_border_width(5);
  $hbox->show;
  $file_vbox->pack_start($hbox, 1, 1, 0);  

  my $button;  
  $button = Gtk::Button->new_with_label (" Probe File ");
  $button->show;
  $hbox->pack_start ($button, 1, 1, 0);
  $button->signal_connect ("clicked",
    sub { $self->probe_file }
  );  
  
  $button = Gtk::Button->new_with_label (" View File ");
  $button->show;
  $hbox->pack_start ($button, 1, 1, 0);
  $button->signal_connect ("clicked",
    sub { $self->view_file }
  );
  
  $button = Gtk::Button->new_with_label (" View File (2nd Audio) ");
  $button->show;
  $hbox->pack_start ($button, 1, 1, 0);
  $button->signal_connect ("clicked",
    sub { $self->view_file2 }
  );  
  
  $button = Gtk::Button->new_with_label (" Use File ");
  $button->show;
  $hbox->pack_start ($button, 1, 1, 0);
  $button->signal_connect ("clicked",
    sub { $self->use_file }
  );

  my $filetitle = Video::Menc::Title->new (
    filename => "",
    project => $self->project
  );
  $filetitle->set_type("file");
  $filetitle->set_postcmd("");
  $self->set_file_title($filetitle);
    
  return $vbox;
}

sub init_audio_popup2 {
  my $self = shift; $self->trace_in;
  
  my $title = $self->selected_title;
  
  return if not $title;

  if ( $title->type eq "file" ) {
    my $audio_popup = $self->gtk_audio_popup2;
    my $audio_popup2 = $self->gtk_audio_popup2_2;
  
    my $audio_popup_menu = Gtk::Menu->new;
    $audio_popup_menu->show;
    $audio_popup->set_menu($audio_popup_menu);
    my $audio_popup_menu2 = Gtk::Menu->new;
    $audio_popup_menu2->show;
    $audio_popup2->set_menu($audio_popup_menu2);

    my $item = Gtk::MenuItem->new ("No Audio");
    $item->show;
    $item->signal_connect ("select", sub {
      $title->set_audio_channel(-1);
      $self->init_title_labels;
      $self->init_mencoder_values;
    } );
    $audio_popup_menu->append($item);
    my $item2 = Gtk::MenuItem->new ("No Audio");
    $item2->show;
    $item2->signal_connect ("select", sub {
      $title->set_audio2_channel(-1);
      $self->init_title_labels;
      $self->init_mencoder_values;
    } );
    $audio_popup_menu2->append($item2);

    if ($title->audio_tracks) {
      my $i = 0;
      foreach my $audio ( @{$title->audio_tracks} ) {
        my $bitrate = $audio->{bitrate};
        $bitrate = $bitrate/1000 if ($bitrate);
        $bitrate = "$bitrate kbits/s " if ($bitrate);
    
        $item = Gtk::MenuItem->new (
          "$audio->{format} $bitrate(Id=$audio->{id})"
        );
        $item->show;
        $item->signal_connect (
          "select", sub {
            $_[1]->set_audio_channel($_[2]);
            $self->init_title_labels;
            $self->init_mencoder_values;
          },
          $title, $i
        );
        $audio_popup_menu->append($item);
        
        $item2 = Gtk::MenuItem->new (
          "$audio->{format} $bitrate(Id=$audio->{id})"
        );
        $item2->show;
        $item2->signal_connect (
          "select", sub {
            $_[1]->set_audio2_channel($_[2]);
            $self->init_title_labels;
            $self->init_mencoder_values;
          },
          $title, $i
        );
        $audio_popup_menu2->append($item2);
        
        ++$i;
      }
    }

    if ($title->audio_channel == -1) {
        $title->set_audio_channel(0) if ($title->audio_tracks); 
    }       
     
    $audio_popup->set_history($title->audio_channel+1);
    $audio_popup2->set_history($title->audio2_channel+1);
  }
  
  1;
}

sub browse_file {
  my $self = shift;
  my $widgets = $self->adjust_widgets;

  my $filename = $widgets->{filename}->get_text;

  $self->show_file_dialog (
    dir      => ".",
    filename => "$filename",
    cb       => sub {
      $self->set_file ( filename => $_[0] );
    },
  );
}

sub set_file {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($filename) = @par{'filename'};

  my $widgets = $self->adjust_widgets;
    $widgets->{filename}->set_text ( $filename );
  $self->use_file;
  
  # update avifile
  my $avifile = $filename;
  
  $avifile =~ s/\.[^\.]+$//;
  my $avifile2 = $avifile.".avi";
  
  $avifile2 = $avifile."_2.avi" if ($avifile2 eq $filename);

  $self->set_avifile ( filename => $avifile2 );  
}

sub use_file {
  my $self = shift; $self->trace_in;
  my $widgets = $self->adjust_widgets;
  my $title = $self->file_title;
  my $filename = $widgets->{filename}->get_text;

  $title->set_filename($filename);
  $title->set_postcmd(" \"$filename\" ");

  $self->set_selected_title($self->file_title);
  
  $self->fill_with_values;  
}

sub probe_file {
  my $self = shift; $self->trace_in;
  my %par = @_;
  my ($no_audio) = @par{'no_audio'};
  
  return if $self->comp('progress')->is_active;
  
  my $title = $self->file_title;
  my $project = $self->project;
  my $content = $project->content;

  $title->set_audio_channel(-1);
  $title->set_audio2_channel(-1);

  my $fh = $content->file_content_async_start (
    filename => $title->filename,
    audioid  => -1,
  );

  $self->comp('progress')->open_continious_progress (
    max_value => 1,
    label     => "Probing File",
    fh        => $fh,
    need_output => 1,
    get_progress_callback => sub {
      my %par = @_;
      my ($buffer) = @par{'buffer'};
      return $buffer;
    },
    finished_callback => sub {
      my %par = @_;
      my  ($output) = @par{'output'};
      
      $content->file_content_async_stop ( 
        fh => $fh,
        output => $output,
        filename => $title->filename,
      );

      $title->set_width ( $content->probe_result->width );
      $title->set_height ( $content->probe_result->height );
      $title->set_fps ( $content->probe_result->fps );
      $title->set_aspect ( $content->probe_result->aspect );
      $title->set_runtime ( $content->probe_result->runtime );
      
      $title->set_audio_tracks ( $content->probe_result->audio_tracks )
        if (not $no_audio);
        
      if ( @{$title->audio_tracks} >= 1 )
      {
        @{$title->audio_tracks}[0]->{bitrate} = 
          $content->probe_result->abitrate;
        @{$title->audio_tracks}[0]->{format} = 
          $content->probe_result->aformat;
      }            

      my $widgets = $self->adjust_widgets;
      my $info = $self->get_title_info (title => $title);  
      
      my $length = $title->runtime;
      my $h = int($length/3600);
      my $m = int(($length-$h*3600)/60);
      my $s = $length-$h*3600-$m*60;
      my $length2 = sprintf ("%02d:%02d:%02d", $h, $m, $s);
      
      my $fileinfo = "${length}sec ($length2), $info"; 
      
        $widgets->{fileinfos}->set_text ( "Fileinfo: $fileinfo" );
  
      $self->set_selected_title( $title );
      $self->fill_with_values;
      
      $self->probe_file2;
    },
    cancel_callback => sub {
      close $fh;
    },
  );
  
  1;
}

sub probe_file2 {  
  my $self = shift; $self->trace_in;

  return if $self->comp('progress')->is_active;

  my $project = $self->project;
  my $content = $project->content;
  my $title = $self->file_title;;
  
  my (@probe_audio);
  my $audios = @{$title->audio_tracks};
  my $i = 0;
    
  if ( $audios > 0 ) {
    for (my $j=0; $j<$audios; $j++) {
      my $audio = $title->audio_tracks->[$j];

      $probe_audio[$i] = {
        filename   => $title->filename,
        audiotrack => $j, 
        audioid    => $audio->{id}
      };
      $i++;        
    }
  }

  if ( @probe_audio ) {
    $self->comp('progress')->open_steps_progress (
      title => "Probe Audiotracks",
      steps => scalar(@probe_audio),
      label => "Probing Audiotrack #",
      need_output => 1,
      open_next_step_callback => sub {
        my %par = @_;
        my ($step) = @par{'step'};

        my $filename = @probe_audio[$step]->{filename};
        my $audiotrack = @probe_audio[$step]->{audiotrack};
        my $audioid = @probe_audio[$step]->{audioid};
      
        return $content->file_content_async_start (
          filename => $filename,
          audioid  => $audioid,
        );
      },
      close_step_callback => sub {
        my %par = @_;
        my  ($step, $fh, $output) =
        @par{'step','fh','output'};

        my $filename = @probe_audio[$step]->{filename};
        my $audiotrack = @probe_audio[$step]->{audiotrack};
        my $audioid = @probe_audio[$step]->{audioid};
            
        eval {
          $content->file_content_async_stop (
            fh     => $fh,
            output => $output,
            filename => $filename,
            audioid  => $audioid,
          );
        };
        if ( not $@ ) {
        
          @{$title->audio_tracks}[$audiotrack]->{bitrate} =
            $content->probe_result->abitrate;
          @{$title->audio_tracks}[$audiotrack]->{format} = 
            $content->probe_result->aformat;

        } else {
          $self->message_window (
            message => "Can't probe Audio-Track #$step\n\n".
                 "Output was:\n\n$output\n\n$@"
          );
        }
      },
      finished_callback => sub {
        $self->set_selected_title( $title );
        $self->fill_with_values;
      }
    );
  }
  
  1;
}

sub view_file {
  my $self = shift;

  my $title = $self->file_title;;

  my $command = $title->precmd." mplayer -osdlevel 3 ".
                $title->audiocmd." ".$title->postcmd." &";

  system ($command);
  
  1;
}

sub view_file2 {
  my $self = shift;

  my $title = $self->file_title;;

  my $command = $title->precmd." mplayer -osdlevel 3 ".
                $title->audiocmd2." ".$title->postcmd." &";

  system ($command);
  
  1;
}  
1;
